"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _utils = require("./utils");
/*
 * Copyright (c) 2018 American Express Travel Related Services Company, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

class Parrot {
  constructor(_scenarios = {}) {
    (0, _defineProperty2.default)(this, "getActiveScenario", () => this.activeScenario);
    (0, _defineProperty2.default)(this, "setActiveScenario", name => {
      this.activeScenario = name;
      _utils.logger.setScenario(name);
    });
    (0, _defineProperty2.default)(this, "getScenarios", () => Object.keys(this.scenarios).map(name => ({
      name,
      mocks: this.scenarios[name]
    })));
    (0, _defineProperty2.default)(this, "setScenarios", scenarios => {
      this.scenarios = (0, _utils.normalizeScenarios)(scenarios);
    });
    (0, _defineProperty2.default)(this, "getScenario", name => this.scenarios[name]);
    (0, _defineProperty2.default)(this, "setScenario", (name, mocks) => {
      const scenarios = {
        [name]: mocks
      };
      this.scenarios = {
        ...this.scenarios,
        ...(0, _utils.normalizeScenarios)(scenarios)
      };
    });
    (0, _defineProperty2.default)(this, "getMock", (name, index) => this.scenarios[name][index]);
    (0, _defineProperty2.default)(this, "setMock", (name, index, mock) => {
      this.scenarios[name][index] = mock;
    });
    (0, _defineProperty2.default)(this, "resolve", async (...platformRequest) => {
      const normalizedRequest = this.normalizeRequest(...platformRequest);
      const resolver = this.resolver(...platformRequest);
      const activeScenarioOverride = this.getActiveScenarioOverride && this.getActiveScenarioOverride(...platformRequest);
      const mocks = this.scenarios[activeScenarioOverride || this.activeScenario];
      const mock = await (0, _utils.matchMock)(normalizedRequest, platformRequest, mocks);
      return (0, _utils.resolveResponse)(normalizedRequest, platformRequest, mock, resolver);
    });
    this.scenarios = (0, _utils.normalizeScenarios)(_scenarios);
    [this.activeScenario] = Object.keys(_scenarios);
    _utils.logger.setScenario(this.activeScenario);
    this.logger = _utils.logger;
  }
}
var _default = exports.default = Parrot;